@@ -46,8 +46,8 @@ async fn main() -> Result<(), Error> {
         let url = addr.clone() + "/runCase?case=" + &case_id.to_string()[..] + "&agent=" + agent;
         println!("{}", url.as_str());
 
-        let client = Client::new(url.as_str(), false);
-        let mut socket = client.connect().await?;
+        let client = Client::new(url.as_str(), true);
+        let mut socket = client.connect2().await?;
 
         while let msg = socket.recv().await {
             let message = match msg {
@@ -88,7 +88,7 @@ async fn get_case_count(addr: String) -> Result<(i32), Error> {
     let url = addr + "/getCaseCount";
 
     let client = Client::new(url.as_str(), true);
-    let mut socket = client.connect().await?;
+    let mut socket = client.connect2().await?;
 
     let mut count = 0;
 
@@ -123,7 +123,7 @@ async fn update_reports(addr: String, agent: &str) -> Result<(), Error> {
     let url = addr + "/updateReports?agent=" + agent;
 
     let client = Client::new(url.as_str(), true);
-    let mut socket = client.connect().await?;
+    let mut socket = client.connect2().await?;
 
     println!("Updating reports...");
 
