@@ -8,7 +8,7 @@ use crate::deflate::Inflator;
 use async_channel::{Sender, Receiver, RecvError};
 use simdutf8::basic::from_utf8;
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 // Message for communication with upstream part of the library.
 pub enum Msg {
     Binary(Vec<u8>),
@@ -47,8 +47,9 @@ impl Msg {
 // of WebSocket (control_tx channel).
 pub struct Reader<R> {
     stream_rx: R,
-    tx: Sender<Msg>,
-    sender: Sender<Msg>,
+    ws_sender: Sender<Msg>,
+    close_send: Sender<u8>,
+    app_sender: Sender<Msg>,
     deflate_supported: bool,
     is_utf8_validate: bool,
     inflator: Inflator,
@@ -63,13 +64,12 @@ impl<R> Reader<R>
     where
         R: AsyncReadExt + std::marker::Unpin + std::marker::Send + 'static,
 {
-    pub fn spawn(stream_rx: R, sender: Sender<Msg>, deflate_supported: bool, is_utf8_validate: bool) -> Receiver<Msg>
-    {
-        let (tx, rx): (Sender<Msg>, Receiver<Msg>) = async_channel::bounded::<Msg>(25);
+    pub fn spawn(stream_rx: R, ws_sender: Sender<Msg>, app_sender: Sender<Msg>, close_send: Sender<u8>, deflate_supported: bool, is_utf8_validate: bool) {
         let mut reader = Reader {
             stream_rx,
-            tx, // output of the messages to the application
-            sender,
+            ws_sender,
+            close_send,
+            app_sender,
             deflate_supported,
             is_utf8_validate,
             inflator: Inflator::new(),
@@ -85,7 +85,6 @@ impl<R> Reader<R>
                 error!("{:?}", e);
             }
         });
-        return rx;
     }
 
     async fn read(&mut self) -> Result<(), Error> {
@@ -102,6 +101,7 @@ impl<R> Reader<R>
                 }
                 Err(e) => {
                     error!("{:?}", e);
+                    self.close_send.send(0).await;
                     break;
                 }
             }
@@ -117,6 +117,7 @@ impl<R> Reader<R>
                     }
                     Err(e) => {
                         error!("{:?}", e);
+                        self.close_send.send(0).await;
                         break;
                     }
                 }
@@ -133,6 +134,7 @@ impl<R> Reader<R>
                     }
                     Err(e) => {
                         error!("{:?}", e);
+                        self.close_send.send(0).await;
                         break;
                     }
                 }
@@ -156,13 +158,16 @@ impl<R> Reader<R>
                             }
                             self.frame.payload.extend_from_slice(&message[0..size]);
                         }
-                        Err(e) => { error!("{:?}", e); }
+                        Err(e) => {
+                            error!("{:?}", e);
+                        }
                     }
                 }
             }
 
             if !self.frame.validate(self.deflate_supported, is_fragment) {
-                self.tx.send(Msg::Close(1002)).await;
+                self.ws_sender.send(Msg::Close(1002)).await;
+                self.close_send.send(0).await;
                 break;
             }
 
@@ -204,6 +209,7 @@ impl<R> Reader<R>
 
             if self.is_utf8_validate {
                 if !self.frame.validate_payload() {
+                    self.close_send.send(0).await;
                     break;
                 }
             }
@@ -219,16 +225,17 @@ impl<R> Reader<R>
             // }
 
             if self.frame.is_text() {
-                self.tx.send(Msg::Text(self.frame.payload.clone())).await;
+                self.ws_sender.send(Msg::Text(self.frame.payload.clone())).await;
             } else if self.frame.is_binary() {
-                self.tx.send(Msg::Binary(self.frame.payload.clone())).await;
+                self.ws_sender.send(Msg::Binary(self.frame.payload.clone())).await;
             } else if self.frame.is_ping() {
-                self.sender.send(Msg::Pong(self.frame.payload.clone())).await;
-                self.tx.send(Msg::Ping(self.frame.payload.clone())).await;
+                self.app_sender.send(Msg::Pong(self.frame.payload.clone())).await;
+                self.ws_sender.send(Msg::Ping(self.frame.payload.clone())).await;
             } else if self.frame.is_close() {
                 let status = self.frame.status();
                 // println!("status {}", status);
-                self.tx.send(Msg::Close(status)).await;
+                self.ws_sender.send(Msg::Close(status)).await;
+                self.close_send.send(0).await;
             }
 
             if self.frame.is_close() {
@@ -429,9 +436,7 @@ impl<W> Writer<W>
     where
         W: AsyncWriteExt + std::marker::Unpin + std::marker::Send + 'static,
 {
-    pub fn spawn(stream_tx: W, mask_frames: bool) -> Sender<Msg> {
-        let (app_tx, app_rx): (Sender<Msg>, Receiver<Msg>) = async_channel::bounded::<Msg>(25);
-
+    pub fn spawn(stream_tx: W, mask_frames: bool, app_rx: Receiver<Msg>) {
         tokio::spawn(async move {
             let mut writer = Writer {
                 stream_tx,
@@ -445,8 +450,6 @@ impl<W> Writer<W>
             }
             trace!("writer loop closed");
         });
-
-        app_tx
     }
 
     async fn run(&mut self) -> Result<(), Error> {
